package java.lang;

import java.math.BigInteger;

public final class Long extends Number implements Comparable<Long>
{
    public static final long MIN_VALUE = -9223372036854775808L;
    public static final long MAX_VALUE = 9223372036854775807L;
    public static final Class<Long> TYPE;
    private final long value;
    public static final int SIZE = 64;
    public static final int BYTES = 8;
    private static final long serialVersionUID = 4290774380558885855L;
    
    public static String toString(long n, int n2) {
        if (n2 < 2 || n2 > 36) {
            n2 = 10;
        }
        if (n2 == 10) {
            return toString(n);
        }
        char[] array = new char[65];
        int n3 = 64;
        int n4 = n < 0L;
        if (n4 == 0) {
            n = -n;
        }
        while (n <= (long)(-n2)) {
            array[n3--] = Integer.digits[(int)(-(n % (long)n2))];
            n /= (long)n2;
        }
        array[n3] = Integer.digits[(int)(-n)];
        if (!(n4 == 0)) {
            array[--n3] = '-';
        }
        return new String(array, n3, 65 - n3);
    }
    
    public static String toUnsignedString(long n, int radix) {
        if (n >= 0L) {
            return toString(n, radix);
        }
        switch (radix) {
            case 2: {
                return toBinaryString(n);
            }
            case 4: {
                return toUnsignedString0(n, 2);
            }
            case 8: {
                return toOctalString(n);
            }
            case 10: {
                long n2 = (n >>> 1) / 5L;
                return toString(n2) + (n - n2 * 10L);
            }
            case 16: {
                return toHexString(n);
            }
            case 32: {
                return toUnsignedString0(n, 5);
            }
            default: {
                return toUnsignedBigInteger(n).toString(radix);
            }
        }
    }
    
    private static BigInteger toUnsignedBigInteger(long val) {
        if (val >= 0L) {
            return BigInteger.valueOf(val);
        }
        return BigInteger.valueOf(Integer.toUnsignedLong((int)(val >>> 32))).shiftLeft(32).add(BigInteger.valueOf(Integer.toUnsignedLong((int)val)));
    }
    
    public static String toHexString(long n) {
        return toUnsignedString0(n, 4);
    }
    
    public static String toOctalString(long n) {
        return toUnsignedString0(n, 3);
    }
    
    public static String toBinaryString(long n) {
        return toUnsignedString0(n, 1);
    }
    
    static String toUnsignedString0(long n, int n2) {
        int max = Math.max((64 - numberOfLeadingZeros(n) + (n2 - 1)) / n2, 1);
        char[] array = new char[max];
        formatUnsignedLong(n, n2, array, 0, max);
        return new String(array, true);
    }
    
    static int formatUnsignedLong(long n, int n2, char[] array, int n3, int n4) {
        int n5 = (1 << n2) - 1;
        do {
            array[n3 + --n4] = Integer.digits[(int)n & n5];
            n >>>= n2;
        } while (n != 0L && n4 > 0);
        return n4;
    }
    
    public static String toString(long n) {
        if (n == Long.MIN_VALUE) {
            return "-9223372036854775808";
        }
        int n2 = (n < 0L) ? (stringSize(-n) + 1) : stringSize(n);
        char[] array = new char[n2];
        getChars(n, n2, array);
        return new String(array, true);
    }
    
    public static String toUnsignedString(long n) {
        return toUnsignedString(n, 10);
    }
    
    static void getChars(long n, int n2, char[] array) {
        int n3 = '\0';
        if (n < 0L) {
            n3 = '-';
            n = -n;
        }
        while (n > 2147483647L) {
            long n4 = n / 100L;
            int n5 = (int)(n - ((n4 << 6) + (n4 << 5) + (n4 << 2)));
            n = n4;
            array[--n2] = Integer.DigitOnes[n5];
            array[--n2] = Integer.DigitTens[n5];
        }
        int i;
        int n6;
        int n7;
        for (i = (int)n; i >= 65536; i = n6, array[--n2] = Integer.DigitOnes[n7], array[--n2] = Integer.DigitTens[n7]) {
            n6 = i / 100;
            n7 = i - ((n6 << 6) + (n6 << 5) + (n6 << 2));
        }
        do {
            int n8 = i * 52429 >>> 19;
            array[--n2] = Integer.digits[i - ((n8 << 3) + (n8 << 1))];
            i = n8;
        } while (i != 0);
        if (n3 != '\0') {
            array[--n2] = n3;
        }
    }
    
    static int stringSize(long n) {
        long n2 = 10L;
        int i = 1;
        while (i < 19) {
            if (n < n2) {
                return i;
            }
            n2 *= 10L;
            ++i;
        }
        return 19;
    }
    
    public static long parseLong(String s, int n) throws NumberFormatException {
        if (s == null) {
            throw new NumberFormatException("null");
        }
        if (n < 2) {
            throw new NumberFormatException("radix " + n + " less than Character.MIN_RADIX");
        }
        if (n > 36) {
            throw new NumberFormatException("radix " + n + " greater than Character.MAX_RADIX");
        }
        long n2 = 0L;
        boolean n3 = false;
        int i = 0;
        int length = s.length();
        long n4 = -9223372036854775807L;
        if (length > 0) {
            int char1 = s.charAt(0);
            if (char1 < '0') {
                if (char1 == '-') {
                    n3 = true;
                    n4 = Long.MIN_VALUE;
                }
                else if (char1 != '+') {
                    throw NumberFormatException.forInputString(s);
                }
                if (length == 1) {
                    throw NumberFormatException.forInputString(s);
                }
                ++i;
            }
            long n5 = n4 / (long)n;
            while (i < length) {
                int digit = Character.digit(s.charAt(i++), n);
                if (digit < 0) {
                    throw NumberFormatException.forInputString(s);
                }
                if (n2 < n5) {
                    throw NumberFormatException.forInputString(s);
                }
                long n6 = n2 * (long)n;
                if (n6 < n4 + (long)digit) {
                    throw NumberFormatException.forInputString(s);
                }
                n2 = n6 - (long)digit;
            }
            return n3 ? n2 : (-n2);
        }
        else {
            throw NumberFormatException.forInputString(s);
        }
    }
    
    public static long parseLong(String s) throws NumberFormatException {
        return parseLong(s, 10);
    }
    
    public static long parseUnsignedLong(String s, int n) throws NumberFormatException {
        if (s == null) {
            throw new NumberFormatException("null");
        }
        int length = s.length();
        if (length > 0) {
            if (s.charAt(0) == '-') {
                throw new NumberFormatException(String.format("Illegal leading minus sign on unsigned string %s.", new Object[] { s }));
            }
            if (length <= 12 || (n == 10 && length <= 18)) {
                return parseLong(s, n);
            }
            long long1 = parseLong(s.substring(0, length - 1), n);
            int digit = Character.digit(s.charAt(length - 1), n);
            if (digit < 0) {
                throw new NumberFormatException("Bad digit at end of " + s);
            }
            long n2 = long1 * (long)n + (long)digit;
            if (compareUnsigned(n2, long1) < 0) {
                throw new NumberFormatException(String.format("String value %s exceeds range of unsigned long.", new Object[] { s }));
            }
            return n2;
        }
        else {
            throw NumberFormatException.forInputString(s);
        }
    }
    
    public static long parseUnsignedLong(String s) throws NumberFormatException {
        return parseUnsignedLong(s, 10);
    }
    
    public static Long valueOf(String s, int n) throws NumberFormatException {
        return valueOf(parseLong(s, n));
    }
    
    public static Long valueOf(String s) throws NumberFormatException {
        return valueOf(parseLong(s, 10));
    }
    
    public static Long valueOf(long n) {
        if (n >= -128L && n <= 127L) {
            return Long.LongCache.cache[(int)n + 128];
        }
        return new Long(n);
    }
    
    public static Long decode(String s) throws NumberFormatException {
        int n = 10;
        int n2 = 0;
        boolean n3 = false;
        if (s.length() == 0) {
            throw new NumberFormatException("Zero length string");
        }
        int char1 = s.charAt(0);
        if (char1 == '-') {
            n3 = true;
            ++n2;
        }
        else if (char1 == '+') {
            ++n2;
        }
        if (s.startsWith("0x", n2) || s.startsWith("0X", n2)) {
            n2 += 2;
            n = 16;
        }
        else if (s.startsWith("#", n2)) {
            ++n2;
            n = 16;
        }
        else if (s.startsWith("0", n2) && s.length() > 1 + n2) {
            ++n2;
            n = 8;
        }
        if (s.startsWith("-", n2) || s.startsWith("+", n2)) {
            throw new NumberFormatException("Sign character in wrong position");
        }
        Long value2;
        try {
            Long value = valueOf(s.substring(n2), n);
            value2 = (n3 ? valueOf(-(long)value) : value);
        }
        catch (NumberFormatException ex) {
            value2 = valueOf(n3 ? ("-" + s.substring(n2)) : s.substring(n2), n);
        }
        return value2;
    }
    
    public Long(long value) {
        this.value = value;
    }
    
    public Long(String s) throws NumberFormatException {
        this.value = parseLong(s, 10);
    }
    
    @Override
    public byte byteValue() {
        return (byte)(int)this.value;
    }
    
    @Override
    public short shortValue() {
        return (short)(int)this.value;
    }
    
    @Override
    public int intValue() {
        return (int)this.value;
    }
    
    @Override
    public long longValue() {
        return this.value;
    }
    
    @Override
    public float floatValue() {
        return (float)this.value;
    }
    
    @Override
    public double doubleValue() {
        return (double)this.value;
    }
    
    @Override
    public String toString() {
        return toString(this.value);
    }
    
    @Override
    public int hashCode() {
        return hashCode(this.value);
    }
    
    public static int hashCode(long n) {
        return (int)(n ^ n >>> 32);
    }
    
    @Override
    public boolean equals(Object o) {
        return o instanceof Long && this.value == (long)o;
    }
    
    public static Long getLong(String s) {
        return getLong(s, null);
    }
    
    public static Long getLong(String s, long n) {
        Long long1 = getLong(s, null);
        return (long1 == null) ? valueOf(n) : long1;
    }
    
    public static Long getLong(String s, Long n) {
        String property = null;
        try {
            property = System.getProperty(s);
        }
        catch (IllegalArgumentException | NullPointerException ex) {}
        if (property != null) {
            try {
                return decode(property);
            }
            catch (NumberFormatException ex2) {}
            return n;
        }
        return n;
    }
    
    @Override
    public int compareTo(Long n) {
        return compare(this.value, n.value);
    }
    
    public static int compare(long n, long n2) {
        return (n < n2) ? -1 : ((n == n2) ? 0 : 1);
    }
    
    public static int compareUnsigned(long n, long n2) {
        return compare(n + Long.MIN_VALUE, n2 + Long.MIN_VALUE);
    }
    
    public static long divideUnsigned(long n, long n2) {
        if (n2 < 0L) {
            return compareUnsigned(n, n2) >= 0?1L:0L;
        }
        if (n > 0L) {
            return n / n2;
        }
        return toUnsignedBigInteger(n).divide(toUnsignedBigInteger(n2)).longValue();
    }
    
    public static long remainderUnsigned(long n, long n2) {
        if (n > 0L && n2 > 0L) {
            return n % n2;
        }
        if (compareUnsigned(n, n2) < 0) {
            return n;
        }
        return toUnsignedBigInteger(n).remainder(toUnsignedBigInteger(n2)).longValue();
    }
    
    public static long highestOneBit(long n) {
        n |= n >> 1;
        n |= n >> 2;
        n |= n >> 4;
        n |= n >> 8;
        n |= n >> 16;
        n |= n >> 32;
        return n - (n >>> 1);
    }
    
    public static long lowestOneBit(long n) {
        return n & -n;
    }
    
    public static int numberOfLeadingZeros(long n) {
        if (n == 0L) {
            return 64;
        }
        int n2 = 1;
        int n3 = (int)(n >>> 32);
        if (n3 == 0) {
            n2 += 32;
            n3 = (int)n;
        }
        if (n3 >>> 16 == 0) {
            n2 += 16;
            n3 <<= 16;
        }
        if (n3 >>> 24 == 0) {
            n2 += 8;
            n3 <<= 8;
        }
        if (n3 >>> 28 == 0) {
            n2 += 4;
            n3 <<= 4;
        }
        if (n3 >>> 30 == 0) {
            n2 += 2;
            n3 <<= 2;
        }
        return n2 - (n3 >>> 31);
    }
    
    public static int numberOfTrailingZeros(long n) {
        if (n == 0L) {
            return 64;
        }
        int n2 = 63;
        int n3 = (int)n;
        int n4;
        if (n3 != 0) {
            n2 -= 32;
            n4 = n3;
        }
        else {
            n4 = (int)(n >>> 32);
        }
        int n5 = n4 << 16;
        if (n5 != 0) {
            n2 -= 16;
            n4 = n5;
        }
        int n6 = n4 << 8;
        if (n6 != 0) {
            n2 -= 8;
            n4 = n6;
        }
        int n7 = n4 << 4;
        if (n7 != 0) {
            n2 -= 4;
            n4 = n7;
        }
        n4 <<= 2;
        if (n4 != 0) {
            n2 -= 2;
        }
        return n2 - (n4 << 1 >>> 31);
    }
    
    public static int bitCount(long n) {
        n -= (n >>> 1 & 0x5555555555555555L);
        n = (n & 0x3333333333333333L) + (n >>> 2 & 0x3333333333333333L);
        n = (n + (n >>> 4) & 0xF0F0F0F0F0F0F0FL);
        n += n >>> 8;
        n += n >>> 16;
        n += n >>> 32;
        return (int)n & 0x7F;
    }
    
    public static long rotateLeft(long n, int n2) {
        return n << n2 | n >>> -n2;
    }
    
    public static long rotateRight(long n, int n2) {
        return n >>> n2 | n << -n2;
    }
    
    public static long reverse(long n) {
        n = ((n & 0x5555555555555555L) << 1 | (n >>> 1 & 0x5555555555555555L));
        n = ((n & 0x3333333333333333L) << 2 | (n >>> 2 & 0x3333333333333333L));
        n = ((n & 0xF0F0F0F0F0F0F0FL) << 4 | (n >>> 4 & 0xF0F0F0F0F0F0F0FL));
        n = ((n & 0xFF00FF00FF00FFL) << 8 | (n >>> 8 & 0xFF00FF00FF00FFL));
        n = (n << 48 | (n & 0xFFFF0000L) << 16 | (n >>> 16 & 0xFFFF0000L) | n >>> 48);
        return n;
    }
    
    public static int signum(long n) {
        return (int)(n >> 63 | -n >>> 63);
    }
    
    public static long reverseBytes(long n) {
        n = ((n & 0xFF00FF00FF00FFL) << 8 | (n >>> 8 & 0xFF00FF00FF00FFL));
        return n << 48 | (n & 0xFFFF0000L) << 16 | (n >>> 16 & 0xFFFF0000L) | n >>> 48;
    }
    
    public static long sum(long n, long n2) {
        return n + n2;
    }
    
    public static long max(long n, long n2) {
        return Math.max(n, n2);
    }
    
    public static long min(long n, long n2) {
        return Math.min(n, n2);
    }
    
    static {
        TYPE = Class.getPrimitiveClass("long");
    }
    
    private static class LongCache
    {
        static final Long[] cache;
        
        static {
            cache = new Long[256];
            for (int i = 0; i < Long.LongCache.cache.length; ++i) {
                Long.LongCache.cache[i] = new Long((long)(i - 128));
            }
        }
    }
}
